package cn.emay;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.google.gson.reflect.TypeToken;

import cn.emay.eucp.inter.framework.dto.CustomFmsIdAndMobile;
import cn.emay.eucp.inter.framework.dto.TemplateResultDto;
import cn.emay.eucp.inter.http.dto.request.FmsBaseRequest;
import cn.emay.eucp.inter.http.dto.request.FmsBatchRequest;
import cn.emay.eucp.inter.http.dto.request.FmsReportRequest;
import cn.emay.eucp.inter.http.dto.request.FmsSingleSendRequest;
import cn.emay.eucp.inter.http.dto.request.FmsTemplateReportRequest;
import cn.emay.eucp.inter.http.dto.request.GetTemplateStatusRequest;
import cn.emay.eucp.inter.http.dto.response.ReportResponse;
import cn.emay.eucp.inter.http.dto.response.SendFmsResponse;
import cn.emay.eucp.inter.http.dto.response.TemplateReportResponse;
import cn.emay.util.AES;
import cn.emay.util.DateUtil;
import cn.emay.util.GZIPUtils;
import cn.emay.util.JsonHelper;
import cn.emay.util.Md5;
import cn.emay.util.http.HttpClient;
import cn.emay.util.http.HttpRequest;
import cn.emay.util.http.HttpRequestBytes;
import cn.emay.util.http.HttpRequestParams;
import cn.emay.util.http.HttpResponseBytes;
import cn.emay.util.http.HttpResponseBytesPraser;
import cn.emay.util.http.HttpResultCode;
import cn.emay.util.http.HttpsRequestBytes;

public class Example {

	// appId
	public static String appId = "EUCP-EMY-FMS1-E9BQE";//
	// 秘钥
	public static String secretKey = "6636933421A24EB0";//

	public static void main(String[] args) {

		//
		// String host = "http://emay.com";//
		String host = "http://127.0.0.1:8999";
		// 时间戳
		String timestamp = DateUtil.toString(new Date(), "yyyyMMddHHmmss");
		String templateInfo = "${name}你好谢谢您的支持";

		String templateId = "";
		Map<String, String> content = new HashMap<String, String>();
		content.put("name", "小强");
		Map<String, String> content1 = new HashMap<String, String>();
		content1.put("name", "小红");

		// 报备模板
		templateId = reportTemplate(host, appId, timestamp, templateInfo);
		System.out.println("templateId: " + templateId);

		// 获取模板状态
		getTemplateStatus(host, appId, templateId);

		// 单条发送
		sendSingleFMS(appId, host, content, "2018010709302911", "15903163405", templateId);

		// 批次发送
		sendBatchFMS(appId, host, new CustomFmsIdAndMobile[] { new CustomFmsIdAndMobile("1", "15903163405", content), new CustomFmsIdAndMobile("2", "15903163405", content1) }, templateId);

		// 获取状态报告
		getFMSReport(appId, host);

		// 获取余额
		getFMSBalance(appId, host);

		/***************************** 加密接口 ****************************/
		// 报备模板
		templateId = reportTemplateSecure(host, appId, secretKey, timestamp, templateInfo);
		System.out.println("templateId: " + templateId);

		// 查询模板状态
		getTemplateStatusSecure(host, appId, secretKey, templateId);

		// 单条发送
		sendSingleFMSSecure(appId, secretKey, host, content, "2018010709302911", "15903163405", templateId);

		// 批次发送
		sendBatchFMSSecure(appId, secretKey, host, new CustomFmsIdAndMobile[] { new CustomFmsIdAndMobile("1", "15903163405", content), new CustomFmsIdAndMobile("2", "15903163405", content1) },
				templateId);

		// 获取状态报告
		getFMSReportSecure(appId, secretKey, host);

		// 获取余额
		getFMSBalanceSecure(appId, secretKey, host);
	}

	private static void getFMSBalanceSecure(String appId, String secretKey, String host) {
		System.out.println("=============begin getFMSBalanceSecure==================");
		FmsBaseRequest request = new FmsBaseRequest();
		request.setAppId(appId);
		request.setRequestTime(System.currentTimeMillis());
		request.setRequestValidPeriod(10);
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("appId", appId);
		headers.put("gzip", "on");
		headers.put("encode", "UTF-8");
		String json = request(request, host + "/fms/secure/getBalance", secretKey, headers, "on", "UTF-8");
		System.out.println("balance: " + json);
		System.out.println("=============end getFMSBalanceSecure==================");
	}

	private static void getFMSBalance(String appId, String host) {
		FmsBaseRequest request = new FmsBaseRequest();
		Long requestTime = System.currentTimeMillis();
		request.setAppId(appId);
		request.setRequestTime(requestTime);
		request.setRequestValidPeriod(10);
		String signStr = appId + secretKey + requestTime;
		String realSign = Md5.md5(signStr.getBytes());
		request.setSign(realSign);
		String json = request(request, host + "/fms/normal/getBalance");
		System.out.println("balance: " + json);
		System.out.println("=============end getFMSBalance==================");
	}

	/**
	 * 
	 * @Title: reportTemplate
	 * @Description: 报备
	 * @param @param host
	 * @param @param appId
	 * @param @param timestamp
	 * @param @param templateInfo
	 * @param @return
	 * @return String
	 */
	private static String reportTemplate(String host, String appId, String timestamp, String templateInfo) {
		FmsTemplateReportRequest requestParams = new FmsTemplateReportRequest();
		Long requestTime = System.currentTimeMillis();
		requestParams.setAppId(appId);
		requestParams.setRequestTime(requestTime);
		requestParams.setRequestValidPeriod(10);
		requestParams.setTemplateInfo(templateInfo);
		String signStr = appId + secretKey + requestTime;
		String realSign = Md5.md5(signStr.getBytes());
		requestParams.setSign(realSign);
		String json = request(requestParams, host + "/fms/normal/templateReport");
		if (json != null) {
			TemplateReportResponse data = JsonHelper.fromJson(new TypeToken<TemplateReportResponse>() {
			}, json);
			System.out.println("reportTemplate result: " + json);
			return data.getTemplateId();
		}
		System.out.println("=============end reportTemplate==================");
		return null;
	}

	/**
	 * 
	 * @Title: reportTemplateSecure
	 * @Description: 加密报备
	 * @param @param host
	 * @param @param appId
	 * @param @param secretKey
	 * @param @param timestamp
	 * @param @param templateInfo
	 * @param @return
	 * @return String
	 */
	private static String reportTemplateSecure(String host, String appId, String secretKey, String timestamp, String templateInfo) {
		System.out.println("=============begin reportTemplateSecure==================");
		FmsTemplateReportRequest request = new FmsTemplateReportRequest();
		request.setAppId(appId);
		request.setRequestTime(System.currentTimeMillis());
		request.setRequestValidPeriod(10);
		request.setTemplateInfo(templateInfo);
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("appId", appId);
		headers.put("gzip", "on");
		headers.put("encode", "UTF-8");
		String json = request(request, host + "/fms/secure/templateReport", secretKey, headers, "on", "UTF-8");
		if (json != null) {
			TemplateReportResponse data = JsonHelper.fromJson(new TypeToken<TemplateReportResponse>() {
			}, json);
			System.out.println("reportTemplateSecure result: " + json);
			return data.getTemplateId();
		}
		System.out.println("=============end reportTemplateSecure==================");
		return null;
	}

	/**
	 * 
	 * @Title: getFMSReport
	 * @Description: 获取状态报告
	 * @param @param appId
	 * @param @param host
	 * @return void
	 */
	private static void getFMSReport(String appId, String host) {
		FmsReportRequest params = new FmsReportRequest();
		Long requestTime = System.currentTimeMillis();
		params.setAppId(appId);
		params.setRequestTime(requestTime);
		params.setRequestValidPeriod(10);
		String signStr = appId + secretKey + requestTime;
		String realSign = Md5.md5(signStr.getBytes());
		params.setSign(realSign);
		String json = request(params, host + "/fms/normal/getFMSReport");
		if (json != null) {
			@SuppressWarnings("unused")
			List<ReportResponse> data = JsonHelper.fromJson(new TypeToken<List<ReportResponse>>() {
			}, json);
			System.out.println("sendBatchFMS result: " + json);
		}
		System.out.println("=============end sendBatchFMS==================");

	}

	private static void getFMSReportSecure(String appId, String secretKey, String host) {
		System.out.println("=============begin getFMSReportSecure==================");
		FmsReportRequest params = new FmsReportRequest();
		params.setAppId(appId);
		params.setRequestTime(System.currentTimeMillis());
		params.setRequestValidPeriod(10);
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("appId", appId);
		headers.put("gzip", "on");
		headers.put("encode", "UTF-8");
		String json = request(params, host + "/fms/secure/getFMSReport", secretKey, headers, "on", "UTF-8");
		if (json != null) {
			@SuppressWarnings("unused")
			List<ReportResponse> data = JsonHelper.fromJson(new TypeToken<List<ReportResponse>>() {
			}, json);
			System.out.println("getFMSReportSecure result: " + json);
		}
		System.out.println("=============end getFMSReportSecure==================");
	}

	/**
	 * 
	 * @Title: sendBatchFMS
	 * @Description: 批次发送
	 * @param @param appId
	 * @param @param host
	 * @param @param customFmsIdAndMobiles
	 * @param @param templateId
	 * @return void
	 */
	private static void sendBatchFMS(String appId, String host, CustomFmsIdAndMobile[] customFmsIdAndMobiles, String templateId) {
		FmsBatchRequest params = new FmsBatchRequest();
		Long requestTime = System.currentTimeMillis();
		params.setAppId(appId);
		params.setFmses(customFmsIdAndMobiles);
		params.setRequestTime(requestTime);
		params.setRequestValidPeriod(10);
		params.setTemplateId(templateId);
		String signStr = appId + secretKey + requestTime;
		String realSign = Md5.md5(signStr.getBytes());
		params.setSign(realSign);
		String json = request(params, host + "/fms/normal/sendBatchFMS");
		if (json != null) {
			@SuppressWarnings("unused")
			SendFmsResponse[] data = JsonHelper.fromJson(new TypeToken<SendFmsResponse[]>() {
			}, json);
			System.out.println("sendBatchFMS result: " + json);
		}
		System.out.println("=============end sendBatchFMS==================");

	}

	private static void sendBatchFMSSecure(String appId, String secretKey, String host, CustomFmsIdAndMobile[] customFmsIdAndMobiles, String templateId) {
		System.out.println("=============begin sendBatchFMSSecure==================");
		FmsBatchRequest params = new FmsBatchRequest();
		params.setAppId(appId);
		params.setFmses(customFmsIdAndMobiles);
		params.setRequestTime(System.currentTimeMillis());
		params.setRequestValidPeriod(10);
		params.setTemplateId(templateId);
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("appId", appId);
		headers.put("gzip", "on");
		headers.put("encode", "UTF-8");
		String json = request(params, host + "/fms/secure/sendBatchFMS", secretKey, headers, "on", "UTF-8");
		if (json != null) {
			@SuppressWarnings("unused")
			SendFmsResponse[] data = JsonHelper.fromJson(new TypeToken<SendFmsResponse[]>() {
			}, json);
			System.out.println("sendBatchFMSSecure result: " + json);
		}
		System.out.println("=============end sendBatchFMSSecure==================");
	}

	/**
	 * 
	 * @Title: sendSingleFMS
	 * @Description: 单条发送
	 * @param @param appId
	 * @param @param host
	 * @param @param string
	 * @param @param string2
	 * @param @param string3
	 * @param @param templateId
	 * @return void
	 */
	private static void sendSingleFMS(String appId, String host, Map<String, String> content, String customFmsId, String mobile, String templateId) {
		FmsSingleSendRequest params = new FmsSingleSendRequest();
		Long requestTime = System.currentTimeMillis();
		params.setAppId(appId);
		params.setContent(content);
		params.setCustomFmsId(customFmsId);
		params.setMobile(mobile);
		params.setRequestTime(requestTime);
		params.setRequestValidPeriod(10);
		params.setTemplateId(templateId);
		String signStr = appId + secretKey + requestTime;
		String realSign = Md5.md5(signStr.getBytes());
		params.setSign(realSign);
		String json = request(params, host + "/fms/normal/sendSingleFMS");
		if (json != null) {
			@SuppressWarnings("unused")
			SendFmsResponse data = JsonHelper.fromJson(new TypeToken<SendFmsResponse>() {
			}, json);
			System.out.println("sendSingleFMS result: " + json);
		}
		System.out.println("=============end sendSingleFMS==================");

	}

	private static void sendSingleFMSSecure(String appId, String secretKey, String host, Map<String, String> content, String customFmsId, String mobile, String templateId) {
		System.out.println("=============begin sendSingleFMSSecure==================");
		FmsSingleSendRequest params = new FmsSingleSendRequest();
		params.setAppId(appId);
		params.setContent(content);
		params.setCustomFmsId(customFmsId);
		params.setMobile(mobile);
		params.setRequestTime(System.currentTimeMillis());
		params.setRequestValidPeriod(10);
		params.setTemplateId(templateId);
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("appId", appId);
		headers.put("gzip", "on");
		headers.put("encode", "UTF-8");
		String json = request(params, host + "/fms/secure/sendSingleFMS", secretKey, headers, "on", "UTF-8");
		if (json != null) {
			@SuppressWarnings("unused")
			SendFmsResponse data = JsonHelper.fromJson(new TypeToken<SendFmsResponse>() {
			}, json);
			System.out.println("sendSingleFMSSecure result: " + json);
		}
		System.out.println("=============end sendSingleFMSSecure==================");
	}

	private static void getTemplateStatus(String host, String appId, String templateId) {
		GetTemplateStatusRequest request = new GetTemplateStatusRequest(templateId);
		request.setAppId(appId);
		Long requestTime = System.currentTimeMillis();
		request.setRequestTime(requestTime);
		request.setRequestValidPeriod(10);
		request.setTemplateId(templateId);
		String signStr = appId + secretKey + requestTime;
		String realSign = Md5.md5(signStr.getBytes());
		request.setSign(realSign);
		String json = request(request, host + "/fms/normal/querytemplate");
		if (json != null) {
			@SuppressWarnings("unused")
			TemplateResultDto data = JsonHelper.fromJson(new TypeToken<TemplateResultDto>() {
			}, json);
			System.out.println("getTemplateStatus result: " + json);
		}
		System.out.println("=============end getTemplateStatus==================");

	}

	private static void getTemplateStatusSecure(String host, String appId, String secretKey, String templateId) {
		System.out.println("=============begin getTemplateStatusSecure==================");
		GetTemplateStatusRequest request = new GetTemplateStatusRequest(templateId);
		request.setAppId(appId);
		request.setRequestTime(System.currentTimeMillis());
		request.setRequestValidPeriod(10);
		request.setTemplateId(templateId);
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("appId", appId);
		headers.put("gzip", "on");
		headers.put("encode", "UTF-8");
		String json = request(request, host + "/fms/secure/querytemplate", secretKey, headers, "on", "UTF-8");
		if (json != null) {
			@SuppressWarnings("unused")
			TemplateResultDto data = JsonHelper.fromJson(new TypeToken<TemplateResultDto>() {
			}, json);
			System.out.println("getTemplateStatus result: " + json);
		}
		System.out.println("=============end getTemplateStatusSecure==================");
	}

	/**
	 * 请求
	 */
	public static String request(Object requestParams, String url) {
		HttpRequest<byte[]> request = null;
		try {
			String requestJson = JsonHelper.toJsonString(requestParams);
			System.out.println("request json: " + requestJson);
			byte[] bytes = requestJson.getBytes("UTF-8");
			System.out.println("request data size : " + bytes.length);
			HttpRequestParams<byte[]> params = new HttpRequestParams<byte[]>();
			params.setCharSet("UTF-8");
			params.setMethod("POST");
			params.setParams(bytes);
			params.setUrl(url);
			if (url.startsWith("https://")) {
				request = new HttpsRequestBytes(params, null);
			} else {
				request = new HttpRequestBytes(params);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		HttpClient client = new HttpClient();
		String code = null;
		String json = null;
		try {
			HttpResponseBytes res = client.service(request, new HttpResponseBytesPraser());
			if (res == null) {
				System.out.println("请求接口异常");
				return null;
			}
			if (res.getResultCode().equals(HttpResultCode.SUCCESS)) {
				if (res.getHttpCode() == 200) {
					code = res.getResultCode().getCode();
					if (code.equals("SUCCESS")) {
						byte[] data = res.getResult();
						System.out.println("response data size : " + data.length);
						json = new String(data, "UTF-8");
						System.out.println("response json: " + json);
					}
				} else {
					System.out.println("请求接口异常,请求码:" + res.getHttpCode());
				}
			} else {
				System.out.println("请求接口网络异常:" + res.getResultCode().getCode());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return json;
	}

	/**
	 * 公共请求方法
	 */
	public static String request(Object requestParams, String url, String secretKey, Map<String, String> headers, String gzip, String encode) {
		HttpRequest<byte[]> request = null;
		try {
			String requestJson = JsonHelper.toJsonString(requestParams);
			System.out.println("request json: " + requestJson);
			if (encode == null || encode.isEmpty()) {
				encode = "UTF-8";
			}
			byte[] bytes = requestJson.getBytes(encode);

			if ("on".equals(gzip)) {
				bytes = GZIPUtils.compress(bytes);
			}
			byte[] aesData = AES.encrypt(bytes, secretKey.getBytes(encode), AES.ALGORITHM_AEPP);
			System.out.println("request data size : " + aesData.length);
			HttpRequestParams<byte[]> params = new HttpRequestParams<byte[]>();
			params.setCharSet(encode);
			params.setMethod("POST");
			params.setParams(aesData);
			params.setUrl(url);
			params.setHeaders(headers);
			if (url.startsWith("https://")) {
				request = new HttpsRequestBytes(params, null);
			} else {
				request = new HttpRequestBytes(params);
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		HttpClient client = new HttpClient();
		String code = null;
		String json = null;
		try {
			HttpResponseBytes res = client.service(request, new HttpResponseBytesPraser());
			if (res == null) {
				System.out.println("请求接口异常");
				return null;
			}
			if (res.getResultCode().equals(HttpResultCode.SUCCESS)) {
				if (res.getHttpCode() == 200) {
					code = res.getHeaders().get("result");
					if (code.equals("SUCCESS")) {
						byte[] data = res.getResult();
						byte[] decData = AES.decrypt(data, secretKey.getBytes(encode), AES.ALGORITHM_AEPP);
						if ("on".equals(gzip)) {
							decData = GZIPUtils.decompress(decData);
						}
						json = new String(decData, encode);
						System.out.println("response data size : " + decData.length);
						System.out.println("response json: " + json);
					}
				} else {
					System.out.println("请求接口异常,请求码:" + res.getHttpCode());
				}
			} else {
				System.out.println("请求接口网络异常:" + res.getResultCode().getCode());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return json;
	}

}
